{ Start of file GetNum.Inc *************************************************}

{ Include GetNum; Get a number for factoring by Multi-precision routines

  Version = 'Version 2.50, last revised: 1994-11-20, 1100 hours';
  Author  = 'Copyright (c) 1981-1994 by author: Harry J Smith,';
  Address = '19628 Via Monte Dr., Saratoga CA 95070.  All rights reserved.';
}
{***************************************************************************}

{ Developed in TURBO Pascal 6.0 }

var
  BadDiv : Boolean;

{--------------------------------------}
procedure Rep( var X, Y, Z : MultiSI); { Z = X Rep Y}
var
  I, J : Integer;
  K    : Mu2Digit;
begin
  Y.GetD( K);
  if K > MaxInt then  K:= MaxInt;
  J:= Trunc( K);  Z.Clear;
  I:= 0;
  while I < J do begin
    I:= I + 1;
    Z.RMul1( 10.0);
    Z.RAdd( X);
    if KeyPressed then  MuInterrupt;
    if KeyHit then begin
      KeyHit:= False;
      Z.ShortWrLn( OutP[0]^, 40);
    end;
    if MuAbort then begin
      Z.Clear;  I:= J;
    end;
  end;
end; { Rep }

{--------------------------------------}
procedure Star( var X, Y, Z : MultiSI); { Z = X (Stat at) Y}
begin
  Start^.SetTo( Y);  Z.SetTo( X);
end; { Star }

{--------------------------------------}
procedure DelBl( var St : String; N : Integer);
  {Delete 1st N characters plus new leading blanks}
begin
  N:= N + 1;
  while Copy( St, N, 1) = ' ' do  N:= N + 1;
  Delete( St, 1, N-1);
end; {DelBl}

{--------------------------------------}
procedure ReportSin( Mess : String; var St : String);
  {Report first syntax condition}
var
  I :Integer;
begin
  if not MuErr then
    for I:= 0 to Echo do begin
      WriteLn( OutP[I]^, Mess, St);
      WriteLn( OutP[I]^);
    end;
  MuErr:= True;
  St:= '';
end; {ReportSin}

{--------------------------------------}
procedure EnterN( var St: String; var X: MultiSI);
  {Set X to operator's input number}
var
  I : Integer;
begin
  X.Value( St, I);
  DelBl( St, I);
end; {EnterN}

{--------------------------------------}
function UnsignedConst( var St : String; var X : MultiSI) : Boolean;
  {Do an Unsigned Constant}
var
  Ch : Char;
begin
  UnsignedConst:= True;
  Ch:= St[1];
  if (0 < Length( St)) and (Ch >= '0') and (Ch <= '9') then
    EnterN( St, X)
  else
    UnsignedConst:= False;
end; {UnsignedConst}

{--------------------------------------}
function Constant( var St : String; var X : MultiSI) : Boolean;
  {Do an constant}
var
  I  : Integer;
  Ch : Char;
begin
  Constant:= True;
  I:= 1;
  if (St[1] = '+') or (St[1] = '-') then  I:= I + 1;
  Ch:= St[I];
  if (I <= Length( St)) and (Ch >= '0') and (Ch <= '9') then
    EnterN( St, X)
  else
    Constant:= False;
end; {Constant}

{--------------------------------------}
function Expression( var St : String; var X : MultiSI) : Boolean;
  {Do an Expression}
  forward;

{--------------------------------------}
function Factor( var St : String; var X : MultiSI) : Boolean; {Do a Factor}
var
  I : Integer;
  T : ^MultiSI;
  Zip : Boolean;
begin
  if St = '' then begin  Factor:= False;  Exit;  end;
  Factor:= True;
  if UnsignedConst( St, X) then  Exit;
  if St[1] = '(' then begin
    DelBl( St, 1);
    if Expression( St,X) then begin
      if St[1] = ')' then  DelBl( St, 1);
      Exit;
    end
    else begin
      ReportSin('Expression expected: (', St);  Exit;
    end;
  end;
  Factor:= False;
end; {Factor}

{--------------------------------------}
function Power( var St : String; var X : MultiSI) : Boolean; {Do a Power}
var
  Sign : String[1];
  I  : Integer;
  Op : Char;
  T  : ^MultiSI;
  T2 : ^MultiSI;
begin
  if not Factor( St, X) then begin  Power:= False;  Exit;  end;
  Power:= True;
  repeat
    Op:= St[1];
    if (Length( St) = 0) or ((Op <> '^') and (Op <> '!')) then  Exit;
    DelBl( St, 1);
    New( T, Init( Size));
    if Op = '!' then begin
      T^.FacMB( X);
      X.SetTo( T^);
    end
    else begin
      Sign:= Copy( St, 1, 1);
      if (Sign = '+') or (Sign = '-') then  DelBl( St, 1)
      else  Sign:= '';
      if Factor( St, T^) then begin
        if Sign = '-' then  T^.Chs;
        New( T2, Init( Size));
        if Op = '^' then  T2^.PowMB( X, T^);
        X.SetTo( T2^);
        Dispose( T2, Done);
      end
      else begin
        ReportSin('Exponent expected: ^' + Sign + '', St);
      end;
    end;
    Dispose( T, Done);
  until False;
end; {Power}

{--------------------------------------}
function Term( var St : String; var X : MultiSI) : Boolean; {Do a Term}
var
  Op : Char;
  T  : ^MultiSI;
  T2 : ^MultiSI;
begin
  Term:= False;
  if not Power( St, X) then  Exit;
  Term:= True;
  repeat
    Op:= St[1];
    if (Length( St) = 0) or ((Op <> '*') and (Op <> '/')) then  Exit;
    DelBl( St, 1);
    New( T, Init( Size));
    if not Power( St, T^) then begin
      ReportSin('Factor expected: ' + Op + '', St);
    end
    else begin
      New( T2, Init( Size));
      if      Op = '*' then  T2^.Mul(X, T^)
      else if Op = '/' then begin
        T2^.DivQR( X, T^, X);
        if Not X.ZTest then BadDiv:= True;
      end;
      X.SetTo( T2^);
      Dispose( T2, Done);
    end;
    Dispose( T, Done);
  until False;
end; {Term}

{--------------------------------------}
function SimpExpr( var St: String; var X: MultiSI): Boolean;
  {Do a Simple Expression}
var
  Sign : String[1];
  Met : Boolean;
  Op : Char;
  T  : ^MultiSI;
begin
  Sign:= Copy( St, 1, 1);
  if (Sign = '+') or (Sign = '-') then  DelBl( St, 1)
  else Sign:= '';
  SimpExpr:= True;
  if not Term( St, X) then begin
    if (Sign = '') then begin  SimpExpr:= False;  Exit;  end;
    ReportSin('Term expected: ' + Sign + '', St);  Exit;
  end;
  if Sign = '-' then  X.Chs;
  repeat
    Op:= St[1];
    if (Length( St) = 0) or ((Op <> '+') and (Op <> '-')) then  Exit;
    DelBl( St, 1);
    New( T, Init( Size));
    if not Term( St, T^) then begin
      ReportSin('Term expected: ' + Op + '', St);
    end
    else begin
      if      Op = '+' then  X.RAdd( T^)
      else if Op = '-' then  X.RSub( T^);
    end;
    Dispose( T, Done);
  until False;
end; {SimpExpr}

{--------------------------------------}
function Expression( var St : String; var X : MultiSI) : Boolean;
  {Do an Expression}
var
  Op : Char;
  T  : ^MultiSI;
  T2 : ^MultiSI;
begin
  Expression:= False;
  if not SimpExpr( St, X) then  Exit;
  Expression:= True;
  repeat
    Op:= St[1];
    if (Length( St) = 0) or ((Op <> 'R') and (Op <> 'S') and (Op <> '`')
                            ) then  Exit;
    DelBl( St, 1);
    New( T, Init( Size));
    if Op = '`' then begin
      T^.FacMB( X);
      X.SetTo( T^);
    end
    else begin
      if not SimpExpr( St, T^) then begin
        ReportSin('Simple Expression expected: ' + Op + '', St);
      end
      else begin
        New( T2, Init( Size));
        if      Op = 'R' then  Rep( X, T^, T2^)
        else if Op = 'S' then Star( X, T^, T2^);
        X.SetTo( T2^);
        Dispose( T2, Done);
      end;
    end;
    Dispose( T, Done);
  until False;
end; {Expression}

{--------------------------------------}
procedure GetNum( var ZIn : MultiSI; Id : String);
var
  I    : Integer;
  ChIn : Char;
  St   : String[128]; { Local operator input string }
  Stat : Boolean;
begin { GetNum }
  repeat
    MuErr:= False;
    BadDiv:= False;
    Start^.SetTo1( 2.0);
    repeat
      for EI:= 0 to Echo do begin
        WriteLn( OutP[EI]^);  WriteLn( OutP[EI]^, Id);
      end;
      for EI:= 0 to Echo do  Write( OutP[EI]^, '? ');
      ReadLn( St);  if Echo = 1 then  WriteLn( OutP[1]^, St);
      DelBl( St, 0);
      St[1]:= UpCase( St[1]);
      if St = '?' then  HelpE;
      if St = 'D' then  ADiagOn:= True;
      if St = 'E' then  TEcho;
      if St = 'Q' then  Fini;
      if St = 'M' then  begin
        NoSieve:= True;  NoMoreDiv:= False;
      end;
      if St = 'N' then  begin
        ADiagOn:= False;  NoSieve:= False;  NoMoreDiv:= False;
      end;
      if St = 'S' then  begin
        NoSieve:= False;  NoMoreDiv:= True;
      end;
      if St = 'T' then  FindUp:= NOT FindUp;
    until (Length( St) > 0) and (IOResult = 0) and
          (St <> '?') and (St <> 'D') and (St <> 'E') and
          (St <> 'Q') and (St <> 'M') and (St <> 'N') and
          (St <> 'S') and (St <> 'T');
    Wr2Ln('');  Diag('Get ready');
    for I:= 1 to Length( St) do begin
      ChIn:= UpCase( St[I]);
      if CHIn = '.' then  ChIn:= '*';
      if CHIn = '=' then  ChIn:= '+';
      if CHIn = 'P' then  ChIn:= '^';
      if CHIn = 'F' then  ChIn:= '!';
      St[I]:= ChIn;
    end;
    Wr2Ln( St);
    StIn:= St;
    Stat:= Good;
    if not Expression( St, ZIn) then  Stat:= Not Good;
    if Length( St) <> 0 then
      ReportSin('Unkown Operation, Command line discarded: ' + '', St);
    if MuErr then  Stat:= Not Good;
    if Stat <> Good then
      for EI:= 0 to Echo do  WriteLn( OutP[EI]^, '?Reenter');
  until Stat = Good;
  for EI:= 0 to Echo do begin
    if BadDiv then
      WriteLn( OutP[EI]^, 'One of your divides did not divide evenly');
    Write( OutP[EI]^, '= ');  ZIn.WritLn( OutP[EI]^);
    WriteLn( OutP[EI]^);
  end;
end; { GetNum }

{ End of file GetNum.Inc ***************************************************}
